package jalview.gui;

import jalview.api.AlignmentViewPanel;
import jalview.datamodel.PDBEntry;
import jalview.datamodel.SequenceI;
import jalview.ext.rbvi.chimera.JalviewChimeraBinding;
import jalview.structure.StructureSelectionManager;

public class JalviewChimeraBindingModel extends JalviewChimeraBinding
{
  private ChimeraViewFrame cvf;

  private FeatureRenderer fr = null;

  public JalviewChimeraBindingModel(ChimeraViewFrame chimeraViewFrame,
          StructureSelectionManager ssm, PDBEntry[] pdbentry,
          SequenceI[][] sequenceIs, String[][] chains, String protocol)
  {
    super(ssm, pdbentry, sequenceIs, chains, protocol);
    cvf = chimeraViewFrame;
  }

  @Override
  public FeatureRenderer getFeatureRenderer(
          AlignmentViewPanel alignment)
  {
    AlignmentPanel ap = (alignment == null) ? cvf.getAlignmentPanel()
            : (AlignmentPanel) alignment;
    if (ap.av.isShowSequenceFeatures())
    {
      if (fr == null)
      {
        fr = (jalview.gui.FeatureRenderer) ap.cloneFeatureRenderer();
      }
      else
      {
        ap.updateFeatureRenderer(fr);
      }
    }

    return fr;
  }

  @Override
  public jalview.api.SequenceRenderer getSequenceRenderer(
          AlignmentViewPanel alignment)
  {
    return new SequenceRenderer(((AlignmentPanel) alignment).av);
  }

  @Override
  public void refreshGUI()
  {
    javax.swing.SwingUtilities.invokeLater(new Runnable()
    {
      public void run()
      {
        cvf.updateTitleAndMenus();
        cvf.revalidate();
      }
    });
  }

  public void updateColours(Object source)
  {
    AlignmentPanel ap = (AlignmentPanel) source;
    // ignore events from panels not used to colour this view
    if (!cvf.isUsedforcolourby(ap))
    {
      return;
    }
    if (!isLoadingFromArchive())
    {
      colourBySequence(ap.av.isShowSequenceFeatures(), ap);
    }
  }

  @Override
  public void releaseReferences(Object svl)
  {
  }

  @Override
  protected void releaseUIResources()
  {
  }

  @Override
  public void refreshPdbEntries()
  {
  }

  /**
   * Send an asynchronous command to Chimera, in a new thread, optionally with
   * an 'in progress' message in a progress bar somewhere
   */
  protected void sendAsynchronousCommand(final String command,
          final String progressMsg)
  {
    Thread thread = new Thread(new Runnable()
    {

      @Override
      public void run()
      {
    long stm = cvf.startProgressBar(progressMsg);
    try
    {
      sendChimeraCommand(command, false);
    } finally
    {
      cvf.stopProgressBar(null, stm);
    }
      }
    });
  thread.start();

  }
}
